#!/bin/bash -u
FNALSYSTEMS="131.225.* 2620:006A:0000:* 2001:0400:2410:* *.fnal.gov *soudan.org"
DEBUG='FALSE'
SSH_CONFIG='/etc/ssh/ssh_config'
DEFAULT_HOST='TRUE'
REPLACE='fnal.gov'

# setup args in the right order for making getopt evaluation
# nice and easy.  You'll need to read the manpages for more info
args=$(getopt -o hdlxs:f:r: -- "$@")
eval set -- "$args"

###########################################################
usage() {
    echo ''           >&2
    echo "$0 [-dxl] [-s <system>] [-f /etc/ssh/ssh_config] [-r fnal.gov]" >&2
    echo "   Change /etc/ssh/ssh_config to meet Fermilab's standards"   >&2
    echo ''                                                             >&2
    echo '     -d enable debugging outupt'                              >&2
    echo '     -r remove settings for FNAL based on -r match'           >&2
    echo '     -s specify the system to be considered'                  >&2
    echo '          NOTE: this replaces the default list'               >&2
    echo '     -f use this ssh_config file'                             >&2
    echo '     -x do not also update the "*" (aka default) Host'        >&2
    echo '     -r replace the match entry containing this value'        >&2
    echo '          NOTE: if the entry is not found one will be created'>&2
    echo " I believe the FNAL systems are:"                             >&2
    for sys in ${FNALSYSTEMS}; do
        echo "    - ${sys}"                                             >&2
    done
    echo ''                                                             >&2
    echo "  Examples:"                                                  >&2
    echo "    $0 -s 131.225.*"                                          >&2
    echo "    $0 -s 131.225.* -s 10.*"                                  >&2
    echo "    $0 -s 131.225.* -s 10.* -f /my/config"                    >&2
    exit 1
}
###########################################################
remove() {
    ########################################################################
    # See if the entry exists
    cat > ${TMPFILE} <<EOF
match /files${SSH_CONFIG}/Host[ . =~ regexp(".*${REPLACE}.*") ]
EOF

    if [[ "${DEBUG}" == 'TRUE' ]]; then
        echo "Finding a entry"
        cat ${TMPFILE}
    fi
    MATCHES=$(augtool -t "Ssh incl ${SSH_CONFIG}" <${TMPFILE})
    echo ${MATCHES} | grep -q '(no matches)'
    if [[ $? -ne 0 ]]; then
        XPATH=$(echo ${MATCHES} | cut -d= -f1)
        cat > ${TMPFILE} <<EOF
rm ${XPATH}

save
EOF
        if [[ "${DEBUG}" == 'TRUE' ]]; then
            echo "Removing existing stanza-> ${MATCHES}"
            cat ${TMPFILE}
        fi
        augtool -t "Ssh incl ${SSH_CONFIG}" >/dev/null <${TMPFILE}
    else
        if [[ "${DEBUG}" == 'TRUE' ]]; then
            echo "No stanza found, nothing to do"
        fi
    fi
}

###########################################################
IFS="
"

REMOVE='FALSE'
READ_HOSTFILE='FALSE'
HOSTFILE=$(mktemp)
for arg in $@; do
    case $1 in
        -- )
            ###########################################################
            # end of getopt args, shift off the -- and get out of the loop
            shift
            break 2
           ;;
         -d )
            ###########################################################
            # enable debugging
            DEBUG='TRUE'
            shift
           ;;
         -r )
            ###########################################################
            # remove config
            REMOVE='TRUE'
            shift
           ;;
         -x )
            ###########################################################
            # disable the '*' host
            DEFAULT_HOST='FALSE'
            shift
           ;;
         -f )
            ###########################################################
            # which config file?
            SSH_CONFIG=$(readlink -f $2)
            shift
            shift
           ;;
         -r )
            ###########################################################
            # replace what entry
            REPLACE=$2
            shift
            shift
           ;;
         -s )
            ###########################################################
            # what host (list)?
            READ_HOSTFILE='TRUE'
            echo $2 >> ${HOSTFILE}
            shift
            shift
           ;;
         -h )
            ###########################################################
            # get help
            usage
           ;;
    esac
done

if [[ "${DEBUG}" == 'TRUE' ]]; then
    set -x
fi

which augtool >/dev/null 2>&1
if [[ $? -ne 0 ]]; then
    echo "Could not find augtool" >&2
    exit 1
fi

if [[ "${READ_HOSTFILE}" == 'TRUE' ]]; then
    FNALSYSTEMS=$(cat ${HOSTFILE} | tr '\012' ' ')
fi
rm -f ${HOSTFILE}

########################################################################
if [[ -f ${SSH_CONFIG} ]]; then
    cp -p ${SSH_CONFIG} ${SSH_CONFIG}.rpmsave
else
    echo "No such file ${SSH_CONFIG}" >&2
    exit 1
fi

TMPFILE=$(mktemp)

if [[ "${REMOVE}" == 'TRUE' ]]; then
    remove
    rm -f ${TMPFILE}
    exit 0
fi

########################################################################
# Setup default Host entry
if [[ "${DEFAULT_HOST}" == 'TRUE' ]]; then
    cat > ${TMPFILE} <<EOF
set /files${SSH_CONFIG}/Host[ . = '*' ]/Protocol 2
set /files${SSH_CONFIG}/Host[ . = '*' ]/GSSAPIAuthentication yes
set /files${SSH_CONFIG}/Host[ . = '*' ]/GSSAPIDelegateCredentials yes
set /files${SSH_CONFIG}/Host[ . = '*' ]/GSSAPIKeyExchange yes
set /files${SSH_CONFIG}/Host[ . = '*' ]/ForwardX11 yes
set /files${SSH_CONFIG}/Host[ . = '*' ]/ForwardX11Trusted yes

save
EOF
    if [[ "${DEBUG}" == 'TRUE' ]]; then
        echo "Setting up Host *"
        cat ${TMPFILE}
    fi
    augtool -t "Ssh incl ${SSH_CONFIG}" >/dev/null <${TMPFILE}
fi

########################################################################
# See if the 'replace' entry exists, if not create it
cat > ${TMPFILE} <<EOF
match /files${SSH_CONFIG}/Host[ . =~ regexp(".*${REPLACE}.*") ]
EOF

if [[ "${DEBUG}" == 'TRUE' ]]; then
    echo "Finding an entry to replace"
    cat ${TMPFILE}
fi
MATCHES=$(augtool -t "Ssh incl ${SSH_CONFIG}" <${TMPFILE})
echo ${MATCHES} | grep -q '(no matches)'
if [[ $? -eq 0 ]]; then
    # host must have a sub value, so Protocol 2 is safe
    cat > ${TMPFILE} <<EOF
ins Host before /files${SSH_CONFIG}/Host[ . = '*' ]
set /files${SSH_CONFIG}/Host[ . = '' ] '${FNALSYSTEMS}'
set /files${SSH_CONFIG}/Host[ . = '${FNALSYSTEMS}' ]/Protocol 2

save
EOF
    if [[ "${DEBUG}" == 'TRUE' ]]; then
        echo "No match found, adding node in"
        cat ${TMPFILE}
    fi
    augtool -t "Ssh incl ${SSH_CONFIG}" >/dev/null <${TMPFILE}
else
    XPATH=$(echo ${MATCHES} | cut -d= -f1)
    cat > ${TMPFILE} <<EOF
set ${XPATH} '${FNALSYSTEMS}'

save
EOF
    if [[ "${DEBUG}" == 'TRUE' ]]; then
        echo "Replacing existing stanza-> ${MATCHES}"
        cat ${TMPFILE}
    fi
    augtool -t "Ssh incl ${SSH_CONFIG}" >/dev/null <${TMPFILE}
fi

########################################################################
# Set the options we want
cat > ${TMPFILE} <<EOF
set /files${SSH_CONFIG}/Host[ . = '${FNALSYSTEMS}' ]/Protocol 2
set /files${SSH_CONFIG}/Host[ . = '${FNALSYSTEMS}' ]/GSSAPIAuthentication yes
set /files${SSH_CONFIG}/Host[ . = '${FNALSYSTEMS}' ]/GSSAPIDelegateCredentials yes
set /files${SSH_CONFIG}/Host[ . = '${FNALSYSTEMS}' ]/GSSAPIKeyExchange yes
set /files${SSH_CONFIG}/Host[ . = '${FNALSYSTEMS}' ]/ForwardX11 yes
set /files${SSH_CONFIG}/Host[ . = '${FNALSYSTEMS}' ]/ForwardX11Trusted yes

save
EOF
if [[ "${DEBUG}" == 'TRUE' ]]; then
    echo "Setting up Host ${FNALSYSTEMS}"
    cat ${TMPFILE}
fi
augtool -t "Ssh incl ${SSH_CONFIG}" >/dev/null <${TMPFILE}


########################################################################
# Ensure selinux context is correct for the config now that we are done with it
chmod 644 /etc/ssh/ssh_config >/dev/null 2>&1
restorecon -f /etc/ssh/ssh_config >/dev/null 2>&1

rm -f ${TMPFILE}

